@@ -6,12 +6,11 @@ use MarpaX::Languages::ECMAScript::AST::Util qw/:all/;
 use MarpaX::Languages::ECMAScript::AST::Impl qw//;
 use Log::Any qw/$log/;
 use constant SEARCH_KEYWORD_IN_GRAMMAR => '# DO NOT REMOVE NOR MODIFY THIS LINE';
-
-use Carp qw/croak/;
+use MarpaX::Languages::ECMAScript::AST::Exceptions qw/:all/;
 
 # ABSTRACT: ECMAScript, grammars base package
 
-our $VERSION = '0.004'; # VERSION
+our $VERSION = '0.005'; # VERSION
 
 #
 # Note: because this module is usually subclasses, internal methods are called
@@ -22,13 +21,13 @@ our $VERSION = '0.004'; # VERSION
 sub new {
   my ($class, $grammar, $package, $spec) = @_;
 
-  croak "Missing grammar" if (! defined($grammar));
-  croak "Missing package name" if (! defined($package));
-  croak "Missing ECMAScript specification" if (! defined($spec));
+  InternalError(error => 'Missing grammar') if (! defined($grammar));
+  InternalError(error => 'Missing package name') if (! defined($package));
+  InternalError(error => 'Missing ECMAScript specification') if (! defined($spec));
 
   my $self  = {
       _content => $grammar,
-      _grammar_option => {action_object  => sprintf('%s::%s', $package, 'Actions')},
+      _grammar_option => {bless_package => $package, action_object  => sprintf('%s::%s', $package, 'Actions')},
       _recce_option => {ranking_method => 'high_rule_only'},
       _strict => 0
   };
@@ -38,6 +37,7 @@ sub new {
   #
   my $characterClass = "\\p{MarpaX::Languages::ECMAScript::AST::Grammar::${spec}::CharacterClasses::Is";
   $self->{_content} =~ s/\\p\{Is/$characterClass/g;
+  $self->{_grammar_option}->{source} = \$self->{_content};
 
   bless($self, $class);
 
@@ -100,15 +100,15 @@ sub _callback {
     eval {$line_columnp = lineAndCol($impl)};
     if (! $@) {
       if (defined($originalErrorString) && $originalErrorString) {
-        logCroak("%s\n%s\n\n%s%s", $originalErrorString, $callackErrorString, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl));
+        SyntaxError(error => sprintf("%s\n%s\n\n%s%s", $originalErrorString, $callackErrorString, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl)));
       } else {
-        logCroak("%s\n\n%s%s", $callackErrorString, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl));
+        SyntaxError(error => sprintf("%s\n\n%s%s", $callackErrorString, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl)));
       }
     } else {
       if (defined($originalErrorString) && $originalErrorString) {
-        logCroak("%s\n%s\n%s", $originalErrorString, $callackErrorString, _context($self, $impl));
+        SyntaxError(error => sprintf("%s\n%s\n%s", $originalErrorString, $callackErrorString, _context($self, $impl)));
       } else {
-        logCroak("%s\n%s", $callackErrorString, _context($self, $impl));
+        SyntaxError(error => sprintf("%s\n%s", $callackErrorString, _context($self, $impl)));
       }
     }
   }
@@ -129,13 +129,17 @@ sub parse {
   my $endp = $optionsp->{end};
   my $endargsp = $optionsp->{endargs} // [];
   my @endargs = @{$endargsp};
+  my $keepOriginalSource = $optionsp->{keepOriginalSource} // 1;
+
   $start //= 0;
   $length //= -1;
 
   #
   # This will create a new instance of the string
   #
-  $source .= ' ';
+  if (! $keepOriginalSource) {
+      $source .= ' ';
+  }
 
   my $pos = $start;
   my $max = length($source) - $start + $length;
@@ -156,7 +160,7 @@ sub parse {
       $pos = _callback($self, $source, $pos, $max, $impl, $failurep, $@, @failureargs);
     } else {
       my $line_columnp = lineAndCol($impl);
-      logCroak("%s\n\n%s%s", $@, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl));
+      SyntaxError(error => sprintf("%s\n\n%s%s", $@, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl)));
     }
   } else {
     $pos = $newpos;
@@ -180,7 +184,7 @@ sub parse {
         $pos = _callback($self, $source, $pos, $max, $impl, $failurep, $@, @failureargs);
       } else {
         my $line_columnp = lineAndCol($impl);
-        logCroak("%s\n\n%s%s", $@, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl));
+        SyntaxError(error => sprintf("%s\n\n%s%s", $@, showLineAndCol(@{$line_columnp}, $source), _context($self, $impl)));
       }
     } else {
       $pos = $newpos;
@@ -201,12 +205,12 @@ sub parse {
 sub value {
   my ($self, $impl) = @_;
 
-  my $rc = $impl->value() || logCroak('%s', _show_last_expression($self, $impl));
+  my $rc = $impl->value() || InternalError(error => sprintf('%s', _show_last_expression($self, $impl)));
   if (! defined($rc)) {
-      croak "Undefined parse tree value";
+      InternalError(error => 'Undefined parse tree value');
   }
   if (defined($impl->value())) {
-      croak "More than one parse tree value\n";
+      InternalError(error => 'More than one parse tree value');
   }
   return ${$rc};
 }
@@ -293,7 +297,7 @@ MarpaX::Languages::ECMAScript::AST::Grammar::Base - ECMAScript, grammars base pa
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 SYNOPSIS
 
@@ -345,35 +349,39 @@ Parse the source given as reference to a scalar, an optional reference to a opti
 
 =item callbackargsp
 
-Callbak Code Reference
+Callbak Code Reference. Default is undef.
 
 =item callbackargs
 
-Reference to an array of Callback Code Reference first arguments
+Reference to an array of Callback Code Reference first arguments. Default is [].
 
 =item failure
 
-Failure callback Code Reference
+Failure callback Code Reference. Default is undef.
 
 =item failureargs
 
-Reference to an array of Failure callback Code Reference first arguments
+Reference to an array of Failure callback Code Reference first arguments. Default is [].
 
 =item end
 
-End callback Code Reference
+End callback Code Reference. Default is undef.
 
 =item endargs
 
-Reference to an array of End callback Code Reference first arguments
+Reference to an array of End callback Code Reference first arguments. Default is [].
+
+=item keepOriginalSource
+
+Because of Automatic Semicolon Insertion that may happen at the end, a space is appended to a copy of the source to be parsed. If a true value, this option disable that append. Default is true.
 
 =back
 
-This method must be called as a super method by grammar using this package as a parent. $self must be a reference to a grammar instantiated via MarpaX::Languages::ECMAScript::AST::Grammar. The callback code will always be called with: per-callback arguments, $source, $pos (i.e. current position), $max (i.e. max position), $impl (i.e. a MarpaX::Languages::ECMAScript::AST::Impl instance). The default and failure callbacks must always return the new position in the stream, and croak if there is an error. In the 'end' and 'failure' callbacks, $pos is not meaningful: this is the last position where external scanning restarted. You might want to look to the getLastLexeme() method. Output of the 'end' callback is ignored.
+This method must be called as a super method by grammar using this package as a parent. $self must be a reference to a grammar instantiated via MarpaX::Languages::ECMAScript::AST::Grammar. The callback code will always be called with: per-callback arguments, $source, $pos (i.e. current position), $max (i.e. max position), $impl (i.e. a MarpaX::Languages::ECMAScript::AST::Impl instance). The default and failure callbacks must always return the new position in the stream, and raise a MarpaX::Languages::ECMAScript::AST::Exception::SyntaxError exception if there is an error. In the 'end' and 'failure' callbacks, $pos is not meaningful: this is the last position where external scanning restarted. You might want to look to the getLastLexeme() method. Output of the 'end' callback is ignored.
 
 =head2 value($self, $impl)
 
-Return the blessed value. $impl is the recognizer instance for the grammar. Will croak if there is more than one parse tree value.
+Return the blessed value. $impl is the recognizer instance for the grammar. Will raise an InternalError exception if there is no parse tree value, or more than one parse tree value.
 
 =head2 getLexeme($self, $lexemeHashp, $impl)
 
